"modsel.rflm" <-
function (object, reduced.model,
                      test=c("Wald", "Deviance")) 
{
    if (!inherits(object, "rlm")) 
        stop("rfmodsel only works for 'rflm' objects")
    test <- match.arg(test)
    X <- model.matrix(object)
    asgn <- attr(X, "assign")
    tt <- terms(object)
    tt0 <- terms(reduced.model)
    tl <- attr(tt, "term.labels")
    tl0 <- attr(tt0, "term.labels")
    numtl0 <- match(tl0 , tl, nomatch=-1)
    if(attr(tt0, "intercept")==1) numtl0 <- c(0, numtl0)
    if(any(is.na(match(numtl0, unique(asgn)))))
      stop("Models are not nested!")
    mod0 <- seq(along = asgn)[!is.na(match(asgn, numtl0))]
    if (length(asgn) == length(mod0)) 
        stop("Models are not strictly nested")
    stopifnot(require("MASS"))
    if(test=="Wald"){
      H0ind <- setdiff(seq(along = asgn), mod0)
      cf <- object$coef[H0ind]
      df <- length(cf)
      t.r <- summary(object, method = "XtWX")
      t.cov <- t.r$cov.unscaled[H0ind, H0ind] * t.r$stddev^2
      t.chisq <- sum(cf * solve(t.cov, cf))
      return(structure(list(statistic=c(chi=t.chisq), df=df,
                            data.name=paste("from", deparse(object$call)),
                            method="robust Wald test", 
                            alternative="two.sided", null.value={
                              c0 <- cf
                              c0[] <- 0
                              c0
                            },
                            p.value=pchisq(t.chisq, df, lower.tail=FALSE)),
                       class="htest"))
    }
    if(test=="Deviance"){
      y <- object$residuals + predict(object)
      s0 <- object$s
      psi <- object$psi
      modfit0 <- rflmFixedScale(x=X[,mod0], y=y, weights=object$weights, 
                     init=object$coef[mod0], psi=psi,  scale=s0)
      SumRhoFull <- sum(psi(object$residuals/s0, rho=TRUE))
      SumRhoMod0 <- sum(psi(modfit0$residuals/s0, rho=TRUE))
      tauStar <- mean(psi(object$residuals/s0, deriv=TRUE)) /
        mean(psi(object$residuals/s0)^2)
      t.chisq <- 2*tauStar*(SumRhoMod0 - SumRhoFull)
      df <- length(asgn)- length(mod0)
      ## only for output - to revise
      H0ind <- setdiff(seq(along = asgn), mod0)
      cf <- object$coef[H0ind]
      ##
      return(structure(list(statistic=c(chisq=t.chisq), df=df,
                            data.name=paste("from", deparse(object$call)),
                            method="robust Deviance test", 
                            alternative="two.sided", null.value={
                              c0 <- cf
                              c0[] <- 0
                              c0
                            },
                            p.value=pchisq(t.chisq, df, lower.tail=FALSE)),
                       class="htest"))

    }
    stop(paste(test, "not yet implemented"))
}

