"print.summary.rfglm" <-
function (x, digits=max(3, getOption("digits") - 3),
                               symbolic.cor=x$symbolic.cor, 
                               signif.stars=getOption("show.signif.stars"),
                               ...) 
{
  cat("\nCall:\n")
  cat(paste(deparse(x$call), sep="\n", collapse="\n"), "\n\n", sep="")
  cat("\nCoefficients:\n")
  coefs <- x$coefficients
  printCoefmat(coefs, digits = digits, signif.stars = signif.stars, 
               na.print = "NA", ...)
  cat("\n(Dispersion parameter for ", x$family$family,
      " family taken to be ", format(x$dispersion), ")\n\n",
      apply(cbind(paste(format.char(c("Null", "Residual"),
                                    width = 8, flag = ""), "deviance:"), 
                  format(unlist(x[c("null.deviance", "deviance")]), 
                         digits=max(5, digits + 1)), " on",
                  format(unlist(x[c("df.null", "df.residual")])),
                  " degrees of freedom\n"), 1, paste, collapse=" "), 
      "\n\n", paste("Method '", x$method, "' used for fitting\n", sep=""),
      "Number of iterations: ", x$iter, "\n")
  correl <- x$correlation
  if (!is.null(correl)) {
    p <- NCOL(correl)
    if (p > 1) {
      cat("\nCorrelation of Coefficients:\n")
      if (is.logical(symbolic.cor) && symbolic.cor) {
        print(symnum(correl, abbr.col=NULL))
      }
      else {
        correl <- format(round(correl, 2), nsmall=2, digits=digits)
        correl[!lower.tri(correl)] <- ""
        print(correl[-1, -p, drop=FALSE], quote=FALSE)
      }
    }
  }
  cat("\n")
  invisible(x)
}

