"rfglm" <-
function (formula, family=binomial, data, weights, subset, 
                   na.action, start=NULL, offset, method="Mqle",
                   weights.on.x = c("none", "hat", "robCov"), control=NULL,  
                   model=TRUE, x = FALSE, y = TRUE, contrasts = NULL,  ...) 
{
  call <- match.call()
  if (is.character(family)) 
    family <- get(family, mode = "function", envir = parent.frame())
  if (is.function(family)) 
    family <- family()
  if (is.null(family$family)) {
    print(family)
    stop("`family' not recognized")
  }
  if (!(family$family %in% c("binomial", "poisson"))){
    if(family$family == "gaussian")
      stop("Use rlm(formula, ...) from library MASS for the Gaussian case")
    else
      stop("Robust fitting method not yet implemented for this family")
  }
  if(is.null(control))
    control <- eval(call(paste("rfglm.", method, ".control", col="", sep="")))
  if (missing(data)) 
    data <- environment(formula)
  ##
  mf <- match.call(expand.dots = FALSE)
  m <- match(c("formula", "data", "subset", "weights", "na.action", 
               "offset"), names(mf), 0)
  mf <- mf[c(1, m)]
  mf$drop.unused.levels <- TRUE
  mf[[1]] <- as.name("model.frame")
  mf <- eval(mf, parent.frame())
  if(method=="model.frame") return(mf)
  mt <- attr(mf, "terms")
  Y <- model.response(mf, "numeric")
  X <- if (!is.empty.model(mt)) 
    model.matrix(mt, mf, contrasts)
  else matrix(, NROW(Y), 0)
  weights <- model.weights(mf)
  offset <- model.offset(mf)
  if (!is.null(weights) && any(weights < 0)) 
    stop("Negative wts not allowed")
  if (!is.null(offset) && length(offset) != NROW(Y)) 
    stop("Number of offsets is ", length(offset), ", should equal ", 
         NROW(Y), " (number of observations)")
  weights.on.x <- match.arg(weights.on.x)
  fit <- switch(method,
                cubif = rfglmCubif(X=X, y=Y, weights=weights, start=start, 
                  offset=offset, family=family,
                   weights.on.x=weights.on.x, control=control,
                  intercept=attr(mt, "intercept") > 0),
                Mqle = rfglmMqle(X=X, y=Y, weights=weights, start=start,  
                  offset=offset, family=family,
                  weights.on.x=weights.on.x, control=control,
                  intercept=attr(mt, "intercept") > 0),
                stop("invalid `method': ", method))
##-     if (any(offset) && attr(mt, "intercept") > 0) {
##-         fit$null.deviance <- glm.fit(x = X[, "(Intercept)", drop = FALSE], 
##-             y = Y, weights = weights, offset = offset, family = family, 
##-             control = control, intercept = TRUE)$deviance
##-     }
  if (model) 
    fit$model <- mf
  fit$na.action <- attr(mf, "na.action")
  if (x) 
    fit$x <- X
  if (!y) 
    fit$y <- NULL
  fit <- c(fit, list(call=call, formula=formula, terms=mt, 
                     data=data, offset=offset, control=control, method=method, 
                     contrasts=attr(X, "contrasts"),
                     xlevels=.getXlevels(mt, mf)))
  class(fit) <- c("rfglm", "glm")
  fit
}

