"rflm.default" <-
function (x, y, weights, ..., w = rep(1, nrow(x)), init = "ls", 
    psi = psi.huber, scale.est = c("MAD", "Huber", "proposal 2"), 
    k2 = 1.345, method = c("M", "MM"), wt.method = c("inv.var", 
        "case"), maxit = 20, acc = 1e-04, test.vec = "resid") 
{
    irls.delta <- function(old, new) sqrt(sum((old - new)^2)/max(1e-20, 
        sum(old^2)))
    irls.rrxwr <- function(x, w, r) {
        w <- sqrt(w)
        max(abs((matrix(r * w, 1, length(r)) %*% x)/sqrt(matrix(w, 
            1, length(r)) %*% (x^2))))/sqrt(sum(w * r^2))
    }
    method <- match.arg(method)
    wt.method <- match.arg(wt.method)
    nmx <- deparse(substitute(x))
    if (is.null(dim(x))) {
        x <- as.matrix(x)
        colnames(x) <- nmx
    }
    else x <- as.matrix(x)
    if (is.null(colnames(x))) 
        colnames(x) <- paste("X", seq(ncol(x)), sep = "")
    if (qr(x)$rank < ncol(x)) 
        stop("x is singular: singular fits are not implemented in rlm")
    if (!(any(test.vec == c("resid", "coef", "w", "NULL")) || 
        is.null(test.vec))) 
        stop("invalid testvec")
    xx <- x
    if (!missing(weights)) {
        if (length(weights) != nrow(x)) 
            stop("Length of weights must equal number of observations")
        if (any(weights < 0)) 
            stop("Negative weights value")
        if (wt.method == "inv.var") {
            fac <- sqrt(weights)
            y <- y * fac
            x <- x * fac
            wt <- NULL
        }
        else {
            w <- w * weights
            wt <- weights
        }
    }
    else wt <- NULL
    if (method == "M") {
        scale.est <- match.arg(scale.est)
        if (!is.function(psi)) 
            psi <- get(psi, mode = "function")
        arguments <- list(...)
        if (length(arguments)) {
            pm <- pmatch(names(arguments), names(formals(psi)), 
                nomatch = 0)
            if (any(pm == 0)) 
                warning(paste("some of ... do not match"))
            pm <- names(arguments)[pm > 0]
            formals(psi)[pm] <- unlist(arguments[pm])
        }
        if (is.character(init)) {
            temp <- if (init == "ls") 
                lm.wfit(x, y, w, method = "qr")
            else if (init == "lts") 
                lqs(x, y, intercept = FALSE, nsamp = 200)
            else stop("init method is unknown")
            coef <- temp$coef
            resid <- temp$resid
        }
        else {
            if (is.list(init)) 
                coef <- init$coef
            else coef <- init
            resid <- y - x %*% coef
        }
    }
    else if (method == "MM") {
        scale.est <- "MM"
        temp <- lqs(x, y, intercept = FALSE, method = "S", k0 = 1.548)
        coef <- temp$coef
        resid <- temp$resid
        psi <- psi.bisquare
        if (length(arguments <- list(...))) 
            if (match("c", names(arguments), nomatch = FALSE)) {
                c0 <- arguments$c
                if (c0 > 1.548) {
                  formals(psi)$c <- c0
                }
                else warning("c must be at least 1.548 and has been ignored")
            }
        scale <- temp$scale
    }
    else stop("method is unknown")
    done <- FALSE
    conv <- NULL
    n1 <- (if (is.null(wt)) 
        nrow(x)
    else sum(wt)) - ncol(x)
    if (scale.est != "MM") 
        scale <- mad(resid, 0)
    theta <- 2 * pnorm(k2) - 1
    gamma <- theta + k2^2 * (1 - theta) - 2 * k2 * dnorm(k2)
    for (iiter in 1:maxit) {
        if (!is.null(test.vec)) 
            testpv <- get(test.vec)
        if (scale.est != "MM") {
            if (scale.est == "MAD") 
                scale <- median(abs(resid))/0.6745
            else scale <- if (is.null(wt)) 
                sqrt(sum(pmin(resid^2, (k2 * scale)^2))/(n1 * 
                  gamma))
            else sqrt(sum(wt * pmin(resid^2, (k2 * scale)^2))/(n1 * 
                gamma))
            if (scale == 0) {
                done <- TRUE
                break
            }
        }
        w <- psi(resid/scale)
        if (!is.null(wt)) 
            w <- w * weights
        temp <- lm.wfit(x, y, w, method = "qr")
        coef <- temp$coef
        resid <- temp$residuals
        if (!is.null(test.vec)) 
            convi <- irls.delta(testpv, get(test.vec))
        else convi <- irls.rrxwr(x, w, resid)
        conv <- c(conv, convi)
        done <- (convi <= acc)
        if (done) 
            break
    }
    if (!done) 
        warning(paste("rlm failed to converge in", maxit, "steps"))
    if (!missing(weights)) {
        tmp <- (weights != 0)
        w[tmp] <- w[tmp]/weights[tmp]
    }
    cl <- match.call()
    cl[[1]] <- as.name("rlm")
    fit <- list(coefficients = coef, residuals = resid, effects = temp$effects, 
        rank = temp$rank, fitted.values = temp$fitted.values, 
        assign = temp$assign, qr = temp$qr, df.residual = NA, 
        w = w, s = scale, psi = psi, k2 = k2, weights = if (!missing(weights)) weights, 
        conv = conv, converged = done, x = xx, call = cl)
    class(fit) <- c("rlm", "lm")
    fit
}

