"rflmFixedScale" <-
function (x, y, weights, w=rep(1, nrow(x)), init, scale, 
                             psi=psi.huber, maxit=20, acc=1e-04,
                             test.vec="resid", wt.method="inv.var")
{
  irls.delta <- function(old, new) sqrt(sum((old - new)^2)/max(1e-20, 
                                                               sum(old^2)))
  irls.rrxwr <- function(x, w, r) {
    w <- sqrt(w)
    max(abs((matrix(r * w, 1, length(r)) %*% x)/
            sqrt(matrix(w, 1, length(r)) %*% (x^2))))/sqrt(sum(w * r^2))
  }
  nmx <- deparse(substitute(x))
  if (is.null(dim(x))) {
    x <- as.matrix(x)
    colnames(x) <- nmx
  }
  else x <- as.matrix(x)
  if (is.null(colnames(x))) 
    colnames(x) <- paste("X", seq(ncol(x)), sep = "")
  if (qr(x)$rank < ncol(x)) 
    stop("x is singular: singular fits are not implemented in rlm")
  if (!(any(test.vec == c("resid", "coef", "w", "NULL")) || 
        is.null(test.vec))) 
    stop("invalid testvec")
  xx <- x
  if (!missing(weights)) {
    if (length(weights) != nrow(x)) 
      stop("Length of weights must equal number of observations")
    if (any(weights < 0)) 
      stop("Negative weights value")
    if (wt.method == "inv.var") {
      fac <- sqrt(weights)
      y <- y * fac
      x <- x * fac
      wt <- NULL
    }
    else {
      w <- w * weights
      wt <- weights
    }
  }
  else wt <- NULL
  ##
  if (is.list(init)) 
    coef <- init$coef
  else coef <- init
  resid <- y - x %*% coef
  if(all(psi(resid) < 0.00001)){
    warning("All robustness weights are 0 in the reduced model - reajusted")
    coef[1] <- coef[1] + median(resid) 
    resid <- y - x %*% coef
  }
    ##
  done <- FALSE
  conv <- NULL
  n1 <- (if (is.null(wt)) 
         nrow(x)
  else sum(wt)) - ncol(x)
  for (iiter in 1:maxit) {
    if (!is.null(test.vec)) 
      testpv <- get(test.vec)
    w <- psi(resid/scale)
    if (!is.null(wt)) 
      w <- w * weights
    temp <- lm.wfit(x, y, w, method = "qr")
    coef <- temp$coef
    resid <- temp$residuals
    if (!is.null(test.vec)) 
      convi <- irls.delta(testpv, get(test.vec))
    else convi <- irls.rrxwr(x, w, resid)
    conv <- c(conv, convi)
    done <- (convi <= acc)
    if (done) 
      break
  }
  if (!done) 
    warning(paste("rlm failed to converge in", maxit, "steps"))
  if (!missing(weights)) {
    tmp <- (weights != 0)
    w[tmp] <- w[tmp]/weights[tmp]
  }
  cl <- match.call()
  cl[[1]] <- as.name("rlm")
  fit <- list(coefficients=coef, residuals=resid, effects=temp$effects, 
              rank=temp$rank, fitted.values=temp$fitted.values, 
              assign=temp$assign, qr=temp$qr, df.residual=NA, 
              w=w, s=scale, psi=psi, weights=if (!missing(weights)) weights, 
              conv=conv, converged=done, x=xx, call=cl)
  class(fit) <- c("rlm", "lm")
  fit
}

