\name{rfnls}
\alias{rfnls}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Robust Fitting of Nonlinear Regression Models}
\description{\code{rfnls} fits a nonlinear regression model by robust
  M-estimators. The fitting is done by iterated reweighted least squares
  (IWLS). 
}
\usage{
rfnls(formula, data, start, weights = NULL, na.action = na.fail,
      psi = psi.huber, test.vec = c("resid", "coef", "w"), maxit = 20,
      acc = 1e-06, algorithm = "default", control = nls.control(),
      trace = FALSE, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{formula}{a symbolic description of the model to be fit
    (cf. \code{\link{nls}}.}
  \item{data}{an optional data frame containing the variables
    in the model.  If not found in \code{data}, the variables are taken
    from \code{environment(formula)}, typically the environment from
    which \code{rfnls} is called.}
  \item{start}{a named numeric vector of starting parameters estimates.}
  \item{weights}{an optional vector of weights to be used in the fitting
    process (for intrinsic weights, not the weights w used in the
    iterative (robust) fit).} 
  \item{na.action}{a function which indicates what should happen when
    the data contain NAs. The default action is for the procedure to
    fail. If NAs are present, use \code{na.exclude} to have residuals with
    length \code{== nrow(data) == length(w)}, where \code{w} are the
    weights used in 
    the iterative robust loop. This is better if the explanatory
    variables in formula are
    time series (and so the NA location is important). For this reason,
    \code{na.omit}, which leads to omission of cases with missing values on any
    required variable, is not suitable here since the residuals length
    is different from \code{nrow(data) == length(w)}. }
  \item{psi}{a function (possibly by name) of the form \code{g(x, 'tuning
    constant(s)', deriv)} that for deriv=0 returns psi(x)/x and for
    deriv=1 returns psi'(x). Tuning constants will be passed in via one
    or several arguments, depending on the psi function. (see also
    \code{\link[MASS]{rlm}}).}  
  \item{test.vec}{character string specifying the convergence
    criterion. The relative change is tested for residuals with a value
    of \code{"resid"} (the default), for coefficients with
    \code{"coef"}, and for weights with \code{"w"}.} 
  \item{maxit}{maximum number of iterations in the robust loop. }
  \item{acc}{convergence tolerance for the robust fit.}
  \item{algorithm}{character string specifying the algorithm to use for
    nls. The default algorithm is a Gauss-Newton algorithm. The other
    alternative is \code{"plinear"}, the Golub-Pereyra algorithm for
    partially linear least-squares models.} 
  \item{control}{an optional list of control settings for
    \code{\link{nls}}. See \code{\link{nls.control}} for the names of
    the settable control values and their effect.} 
  \item{trace}{logical value indicating if a "trace" of the nls
    iteration progress should be printed. Default is \code{FALSE}.  
    If \code{TRUE}, in each robust iteration, the residual sum-of-squares
    and the parameter values are printed at the conclusion of each nls
    iteration. When the "plinear" algorithm is used, the conditional
    estimates of the linear parameters are printed after the nonlinear
    parameters.} 
  \item{\dots}{potentially arguments to be passed to the psi function
    (see above).} 
}
\details{
  This function corresponds to the function \code{\link[sfsmisc]{rnls}} in the
  package \pkg{sfsmisc} but with a formula interface as in
  \code{\link{nls}} and generic accessor functions
  \code{\link{summary}}, \code{\link{coefficients}},
  \code{fitted.values} and \code{residuals}.
}
\value{
  An object of S3 class "nls.robust", inheriting from class "nls", (see
  \code{\link{nls}}). It is a list with components 
  \item{FixIt }{???}

}

%% \note{ ~~further notes~~ }
%% 
%%  ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{\code{\link[sfsmisc]{rnls}}}
\examples{
## ??? To do
}

\keyword{models}
\keyword{regression}
\keyword{nonlinear}
\keyword{robust}
